#include "blocks.h"
#include "renderer.h"
#include "scene.h"
#include "assets/block.h"
#include "sprite.h"
#include "id.h"

#define MAX_BLOCKS  10

#define BLOCK_WIDTH   BLOCKS_BLOCK_W*2
#define BLOCK_HEIGHT  BLOCKS_BLOCK_H

#define BLOCK_CX    4
#define BLOCK_CY    BLOCK_HEIGHT
#define NULL_BLOCK  (void*)0

Block blocks[SCENE_BLOCK_COUNT];
u8    blocks_count;

void blocks_reset() {
  blocks_count = 0;
}

void blocks_add(u16 x, u8 y, bool drawable, bool climbable) {
  Block* block = &blocks[blocks_count++];
  block->id = id_generate();
  block->x0 = x - BLOCK_CX;
  block->y0 = y - BLOCK_CY;
  block->x1 = x + BLOCK_CX;
  block->y1 = y;
  block->drawable = drawable;
  block->climbable = climbable;
}

void blocks_drawer(void* payload, u16 x, u8 y) {
  // Block*  block = (Block*)payload;
  u8*     scr = renderer_getScreenPointer(x, y);
  sprite_drawSprite(blocks_block, scr, BLOCKS_BLOCK_W, BLOCKS_BLOCK_H);
}

void blocks_clear(Block* block) {
  if (block->drawable) {
    renderer_clear(block->id, block->x0, block->y0,
                   BLOCK_WIDTH, BLOCK_HEIGHT);
  }
}

void blocks_render() {
  Block*  block = blocks;
  u8      count = blocks_count;
  while (count--) {
    if (block->drawable &&
        renderer_isVisible(block->x0) &&
        renderer_isVisible(block->x1)) {
      renderer_draw(block->id,
                    block->x0, block->y0,
                    BLOCK_WIDTH, BLOCK_HEIGHT,
                    &blocks_drawer, (void*)block);
    }
    block++;
  }
}
