#include <cpctelera.h>
#include "chapters.h"
#include "entities.h"
#include "fader.h"
#include "print.h"
#include "renderer.h"
#include "scene.h"
#include "player.h"

#define BILLBOARD_TITLE_COLOR     9
#define BILLBOARD_SUBTITLE_COLOR  10

u8 chapters_current;

void chapters_wait() {
  u8  counter = 200;
  while (counter--) {
    __asm__("halt");
    __asm__("halt");
    cpct_waitVSYNC();

    cpct_scanKeyboard();
    if (cpct_isAnyKeyPressed())
      return;
  }
}

void chapters_waitForced() {
  u16 counter = 10 * 25;
  while (counter--) {
    __asm__("halt");
    __asm__("halt");
    cpct_waitVSYNC();
  }
}

u8* getChapter(u8 index) {
  u8* chapter = scene_chapters;
  while (index > 0) {
    while (*(chapter++) != 0);
    index--;
  }
  return chapter;
}

void showBillboard(u8* title, u8* subtitle) {
  u8* scr;

  fader_fadeOut();

  cpct_memset(CPCT_VMEM_START, 0, 0x4000);
  renderer_center(0);

  print_setColor(BILLBOARD_TITLE_COLOR);
  scr = renderer_getScreenPointer(SCREEN_WIDTH / 2, 30);
  print_drawString(scr, title);
  scr = renderer_getScreenPointer(SCREEN_WIDTH / 2, 50);
  print_setColor(BILLBOARD_SUBTITLE_COLOR);
  print_drawString(scr, subtitle);

  fader_fadeIn();
  chapters_wait();
  fader_fadeOut();
}

void chapters_set(u8 index) {
  u8 title[] = {'C', 'H', 'A', 'P', 'T', 'E', 'R', ' ', 0, 0, 0};
  u8* chapter = getChapter(index);
  u8  number = *(chapter++);

  if (number < 10) {
    title[8] = number + '0';
  } else {
    title[8] = (number / 10) + '0';
    title[9] = (number % 10) + '0';
  }

  showBillboard(title, chapter);

  chapters_current = index;
  entities_reset(index);
  renderer_center(player_x);
  renderer_clearAll();

  fader_fadeIn();
}

void chapters_reset() {
  chapters_set(chapters_current);
}

void chapters_next() {
  chapters_set(chapters_current + 1);
}

void chapters_previous() {
  chapters_set(chapters_current - 1);
}

void drawCentered(u8 color, char* string, u8 y) {
  u8*  scr = renderer_getScreenPointer(SCREEN_WIDTH / 2, y);
  print_setColor(color);
  print_drawString(scr, string);
}

void chapters_end() {
  showBillboard("You got the book\0", "End of the first episode\0");

  cpct_memset(CPCT_VMEM_START, 0, 0x4000);

  drawCentered(BILLBOARD_TITLE_COLOR, "PRODUCTION AND CODE\0", 0);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "Tulo\0", 10);

  drawCentered(BILLBOARD_TITLE_COLOR, "CODE\0", 30);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "Tainus\0", 40);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "Gabi\0", 50);

  drawCentered(BILLBOARD_TITLE_COLOR, "ART\0", 70);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "Alberto Hache\0", 80);

  drawCentered(BILLBOARD_TITLE_COLOR, "MUSIC\0", 100);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "Jose A Martin\0", 110);

  drawCentered(BILLBOARD_TITLE_COLOR, "A 2017 Dreamin`Bits production\0", 140);
  drawCentered(BILLBOARD_SUBTITLE_COLOR, "https:^^dreaminbits_com/\0", 150);

  fader_fadeIn();

  chapters_waitForced();
  while (1) {
    cpct_scanKeyboard();
    if (cpct_isAnyKeyPressed()) break;
  }

  chapters_set(0);
}
