#include <cpctelera.h>
#include "guardian.h"
#include "id.h"
#include "player.h"
#include "renderer.h"
#include "assets/guardian.h"
#include "sprite.h"
#include "print.h"
#include "chapters.h"
#include "debug.h"

#define GUARDIAN_COLOR  10
#define GUARDIAN_WIDTH  GUARDIANS_GUARDIAN_0_W * 2
#define GUARDIAN_HEIGHT GUARDIANS_GUARDIAN_0_H
#define GUARDIAN_CX     GUARDIAN_WIDTH / 2
#define GUARDIAN_CY     GUARDIAN_HEIGHT

#define GUARDIAN_CHAPTER  4
#define GUARDIAN_DISTANCE 30
#define GUARDIAN_TTL      100

u8    guardian_id;
u16   guardian_x, guardian_x0, guardian_x1;
u8    guardian_y;
u8*   guardian_sprite;
u8    guardian_ttl;
void  (*guardian_step)();

const u8 guardian_text[] = "Do you have the book?\0";
u8* guardian_char;
u8* guardian_scr;

void guardian_wait();
void guardian_startTalking();
void guardian_talk();
void guardian_stop();
void guardian_doNothing();

void guardian_init() {
  guardian_id = id_generate();
  guardian_sprite = guardians_guardian_0;
  guardian_step = guardian_doNothing;
}

void guardian_start(u16 x, u8 y, u8 chapter) {
  guardian_id = id_generate();
  guardian_sprite = guardians_guardian_0;
  guardian_x = x;
  guardian_x0 = x - GUARDIAN_CX;
  guardian_x1 = x + GUARDIAN_CX;
  guardian_y = y - GUARDIAN_CY;
  guardian_step = chapter == GUARDIAN_CHAPTER? guardian_wait: guardian_doNothing;
}

void guardian_doNothing() {

}

void guardian_wait() {
  if ((player_x - guardian_x1) < GUARDIAN_DISTANCE) {
    guardian_step = guardian_talk;
    guardian_startTalking();
  }
}

void guardian_startTalking() {
  print_setColor(GUARDIAN_COLOR);

  guardian_scr = renderer_getScreenPointer(guardian_x, 40) -
                 print_stringLength(guardian_text);
  guardian_id = id_generate();
  guardian_sprite = guardians_guardian_1;
  guardian_char = guardian_text;
}

void guardian_talk() {
  u8 ch = *(guardian_char++);
  if (ch == 0) {
    guardian_step = guardian_stop;
    guardian_ttl = GUARDIAN_TTL;
  } else {
    print_drawChar(guardian_scr, ch);
    guardian_scr += 2;
  }
}

void guardian_stop() {
  guardian_ttl--;
  if (guardian_ttl == 0) {
    chapters_next();
  }
}

void guardian_drawer(void* payload, u16 x, u8 y) {
  u8* scr = renderer_getScreenPointer(x, y);
  sprite_drawSprite(guardian_sprite, scr,
                    GUARDIANS_GUARDIAN_0_W, GUARDIANS_GUARDIAN_0_H);
}

void guardian_render() {
  if (renderer_isVisible(guardian_x0) && renderer_isVisible(guardian_x1)) {
    renderer_draw(guardian_id,
                  guardian_x0, guardian_y,
                  GUARDIAN_WIDTH, GUARDIAN_HEIGHT,
                  &guardian_drawer, (void*)0);
  }
}
