#include <cpctelera.h>
#include "rain.h"
#include "id.h"
#include "collider.h"
#include "renderer.h"
#include "sprites.h"
#include "player.h"

#define RAIN_MIN_Y  0
#define RAIN_MAX_Y  115

#define RAIN_MIN_VY 3
#define RAIN_MAX_VY 8

#define RAIN_DROP_CX      0
#define RAIN_DROP_CY      RAIN_DROP_HEIGHT

#define RAIN_SPLASH_CX    2
#define RAIN_SPLASH_CY    RAIN_SPLASH_HEIGHT
#define RAIN_SPLASH_TTL   80

#define RAIN_DROPS_COUNT  4

#define FY_SCALE  3

typedef struct {
  u8  id;
  u16 x;
  u8  vy, y, yMax;
  u8  cx, cy, width, height;
  u8  sprite;
  u8  ttl;
} RainDrop;

RainDrop rain_drops[RAIN_DROPS_COUNT];
Collider rain_collider;
bool     rain_isActive;

void rain_dropReset(RainDrop* drop);

void rain_reset() {
  rain_isActive = false;
  collider_create(&rain_collider);
}
void rain_start() {
  RainDrop* drop = rain_drops;

  for (u8 i = 0; i < RAIN_DROPS_COUNT; i++, drop++) {
    drop->id = id_generate();
    rain_dropReset(drop);
  }

  rain_isActive = true;
}

void rain_dropReset(RainDrop* drop) {
  u16 x = (player_x - SCREEN_WIDTH / 2) + (cpct_rand8() % SCREEN_WIDTH);
  u8  vy = RAIN_MIN_VY + (cpct_rand8() % (RAIN_MAX_VY - RAIN_MIN_VY));
  collider_check(&rain_collider, x, RAIN_MIN_Y, x, RAIN_MAX_Y);
  drop->x = x;
  drop->y = RAIN_DROP_HEIGHT;
  drop->yMax = FIXED2INT(rain_collider.fy, FY_SCALE);
  drop->vy = vy;
  drop->cx = RAIN_DROP_CX;
  drop->cy = RAIN_DROP_CY;
  drop->width = RAIN_DROP_WIDTH;
  drop->height = RAIN_DROP_HEIGHT;
  drop->sprite = RAIN_DROP_SPRITE_0;
  drop->ttl = RAIN_SPLASH_TTL;
}

void rain_clear() {
  RainDrop* drop = rain_drops;

  if (!rain_isActive) return;

  for (u8 i = 0; i < RAIN_DROPS_COUNT; i++, drop++) {
    if (renderer_isVisible(drop->x)) {
      renderer_clear(drop->id, drop->x - drop->cx, drop->y - drop->cy,
                     drop->width, drop->height);
    }
  }
}

void rain_drawSprite(void* payload, u16 x, u8 y) {
  RainDrop* drop = payload;
  u8*       scr = renderer_getScreenPointer(x, y);
  sprites_drawers[drop->sprite](scr);
}

void rain_render() {
  RainDrop* drop = rain_drops;

  if (!rain_isActive) return;

  for (u8 i = 0; i < RAIN_DROPS_COUNT; i++, drop++) {
    if (renderer_isVisible(drop->x)) {
      renderer_draw(drop->id,
                    drop->x - drop->cx, drop->y - drop->cy,
                    drop->width, drop->height,
                    &rain_drawSprite, (void*)drop);
    }
  }
}
void rain_step() {
  RainDrop* drop = rain_drops;

  if (!rain_isActive) return;

  for (u8 i = 0; i < RAIN_DROPS_COUNT; i++, drop++) {
    if (drop->y < drop->yMax) {
      drop->y += drop->vy;
      if (drop->y >= drop->yMax) {
        drop->y = drop->yMax;
        drop->sprite = RAIN_SPLASH_SPRITE_0;
        drop->cx = RAIN_SPLASH_CX;
        drop->cy = RAIN_SPLASH_CY;
        drop->width = RAIN_SPLASH_WIDTH;
        drop->height = RAIN_SPLASH_HEIGHT;
      }
    } else if (drop->ttl) {
      drop->ttl--;
    } else {
      renderer_forceRedraw(drop->id);
      rain_dropReset(drop);
    }
  }
}
